@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -19,20 +19,19 @@
 
 ===========================================================================*/
 
-#include <getfemint.h>
-#include <getfemint_mesh_im.h>
-#include <getfemint_models.h>
-#include <getfem/getfem_assembling.h>
-#include <getfem/getfem_level_set.h>
-#include <getfemint_misc.h>
-#include <getfemint_gsparse.h>
 #include <getfem/getfem_interpolation.h>
 #include <getfem/getfem_nonlinear_elasticity.h>
+#include <getfem/getfem_plasticity.h>
 #include <getfem/getfem_fourth_order.h>
 #include <getfem/getfem_contact_and_friction_nodal.h>
 #include <getfem/getfem_contact_and_friction_integral.h>
 #include <getfem/getfem_contact_and_friction_large_sliding.h>
 #include <getfem/getfem_generic_assembly.h>
+#include <getfem/getfem_level_set.h>
+#include <getfem/getfem_im_data.h>
+#include <getfem/getfem_models.h>
+#include <getfemint_misc.h>
+#include <getfemint_gsparse.h>
 
 #if GETFEM_HAVE_METIS_OLD_API
 extern "C" void METIS_PartGraphKway(int *, int *, int *, int *, int *, int *,
@@ -364,14 +363,14 @@ do_generic_assembly(mexargs_in& in, mexargs_out& out, bool on_boundary)
   std::string s = in.pop().to_string();
   getfem::generic_assembly assem(s);
   /* stores the mesh_fem identifiers */
-  while (in.remaining() && in.front().is_mesh_im()) {
-    assem.push_mi(*in.pop().to_mesh_im());
+  while (in.remaining() && is_meshim_object(in.front())) {
+    assem.push_mi(*to_meshim_object(in.pop()));
   }
   if (assem.im().size() == 0)
     THROW_BADARG("generic assembly without any mesh_im has no sense!");
   /* stores the mesh_fem identifiers */
-  while (in.remaining() && in.front().is_mesh_fem()) {
-    const getfem::mesh_fem *mf = in.pop().to_const_mesh_fem();
+  while (in.remaining() && is_meshfem_object(in.front())) {
+    const getfem::mesh_fem *mf = to_meshfem_object(in.pop());
     if (!mesh) mesh = &(mf->linked_mesh());
     assem.push_mf(*mf);
   }
@@ -393,11 +392,13 @@ do_generic_assembly(mexargs_in& in, mexargs_out& out, bool on_boundary)
   mesh->intersect_with_mpi_region(rg);
   assem.assembly(rg);
   // get the matrix back
+  int nb_mat = 0;
   for (size_type i=0; out.remaining() && i < assem.mat().size(); ++i) {
     if (assem.mat()[i] != 0) {
-      getfem::base_asm_mat *BM = assem.mat()[i];
-      getfem::asm_mat<gf_real_sparse_by_col> * M =
-        static_cast<getfem::asm_mat<gf_real_sparse_by_col>*>(BM);
+      nb_mat++;
+      getfem::base_asm_mat *BM = assem.mat()[i].get();
+      getfem::asm_mat<gf_real_sparse_by_col> *M =
+        static_cast<getfem::asm_mat<gf_real_sparse_by_col> *>(BM);
       out.pop().from_sparse(*M->mat());
     }
   }
@@ -405,7 +406,7 @@ do_generic_assembly(mexargs_in& in, mexargs_out& out, bool on_boundary)
   if (out.remaining()) {
     for (size_type i=0; out.remaining() && i < assem.vec().size(); ++i) {
       if (assem.vec()[i] != 0) {
-        getfem::base_asm_vec *BV = assem.vec()[i];
+        getfem::base_asm_vec *BV = assem.vec()[i].get();
         getfem::asm_vec<darray_with_gfi_array> *V =
           static_cast<getfem::asm_vec<darray_with_gfi_array> *>(BV);
         mexarg_out mo = out.pop(); mo.arg = V->vec()->mx;
@@ -417,16 +418,15 @@ do_generic_assembly(mexargs_in& in, mexargs_out& out, bool on_boundary)
 
 static void do_high_level_generic_assembly(mexargs_in& in, mexargs_out& out) {
 
-  getfemint_mesh_im *gfi_mim = in.pop().to_getfemint_mesh_im();
+  getfem::mesh_im *mim = to_meshim_object(in.pop());
   size_type order = in.pop().to_integer(0, 2);
   std::string expr = in.pop().to_string();
   size_type region = in.pop().to_integer();
 
   getfem::ga_workspace workspace1;
   getfem::model dummy_md;
-  bool with_model = in.remaining() && in.front().is_model();
-  const getfem::model &md = with_model ? in.pop().to_getfemint_model()->model()
-                                       : dummy_md;
+  bool with_model = in.remaining() && is_model_object(in.front());
+  const getfem::model &md = with_model ? *to_model_object(in.pop()) : dummy_md;
   getfem::ga_workspace workspace2(md);
   getfem::ga_workspace &workspace = with_model ? workspace2 : workspace1;
 
@@ -439,10 +439,10 @@ static void do_high_level_generic_assembly(mexargs_in& in, mexargs_out& out) {
     bool is_cte = (in.pop().to_integer() == 0);
     const getfem::mesh_fem *mf(0);
     const getfem::im_data *mimd(0);
-    if (in.front().is_mesh_fem())
-      mf = in.pop().to_const_mesh_fem();
-    else if (in.front().is_mesh_im_data())
-      mimd = in.pop().to_const_mesh_im_data();
+    if (is_meshfem_object(in.front()))
+      mf = to_meshfem_object(in.pop());
+    else if (is_meshimdata_object(in.front()))
+      mimd = to_meshimdata_object(in.pop());
     darray U = in.pop().to_darray();
     GMM_ASSERT1(vectors.find(varname) == vectors.end(),
                 "The same variable/constant name is repeated twice: "
@@ -475,8 +475,7 @@ static void do_high_level_generic_assembly(mexargs_in& in, mexargs_out& out) {
   }
 
   size_type add_derivative_order = (order != 0) ? 2 : 0;
-  workspace.add_expression(expr, gfi_mim->mesh_im(), region,
-                           add_derivative_order);
+  workspace.add_expression(expr, *mim, region, add_derivative_order);
 
   switch (order) {
   case 0:
@@ -512,21 +511,20 @@ static void do_expression_analysis(mexargs_in& in) {
 
   std::string expr = in.pop().to_string();
 
-  bool with_mesh = in.remaining() && in.front().is_mesh();
-  bool with_mim = !with_mesh && in.remaining() && in.front().is_mesh_im();
+  bool with_mesh = in.remaining() && has_mesh_object(in.front());
+  bool with_mim = !with_mesh && in.remaining() && is_meshim_object(in.front());
 
   getfem::mesh dummy_mesh;
-  const getfem::mesh_im dummy_mim(with_mesh ? *(in.pop().to_mesh()) : dummy_mesh);
-  const getfem::mesh_im &mim = with_mim ? *(in.pop().to_const_mesh_im())
+  const getfem::mesh_im dummy_mim(with_mesh ? *(extract_mesh_object(in.pop())):dummy_mesh);
+  const getfem::mesh_im &mim = with_mim ? *(to_meshim_object(in.pop()))
                                         : dummy_mim;
 
   size_type der_order = in.remaining() && in.front().is_integer()
                       ? in.pop().to_integer(0, 2) : 0;
   getfem::ga_workspace workspace1;
   getfem::model dummy_md;
-  bool with_model = in.remaining() && in.front().is_model();
-  const getfem::model &md = with_model ? in.pop().to_getfemint_model()->model()
-                                       : dummy_md;
+  bool with_model = in.remaining() && is_model_object(in.front());
+  const getfem::model &md = with_model ? *to_model_object(in.pop()) : dummy_md;
   getfem::ga_workspace workspace2(md);
   getfem::ga_workspace &workspace = with_model ? workspace2 : workspace1;
 
@@ -542,10 +540,10 @@ static void do_expression_analysis(mexargs_in& in) {
     bool is_cte(in.pop().to_integer() == 0);
     const getfem::mesh_fem *mf(0);
     const getfem::im_data *mimd(0);
-    if (in.remaining() && in.front().is_mesh_fem())
-      mf = in.pop().to_const_mesh_fem();
-    else if (in.remaining() && in.front().is_mesh_im_data())
-      mimd = in.pop().to_const_mesh_im_data();
+    if (in.remaining() && is_meshfem_object(in.front()))
+      mf = to_meshfem_object(in.pop());
+    else if (in.remaining() && is_meshimdata_object(in.front()))
+      mimd = to_meshimdata_object(in.pop());
     GMM_ASSERT1(varnames.count(varname) == 0,
                 "The same variable/constant name is repeated twice: "
                  << varname)
@@ -569,8 +567,7 @@ static void do_expression_analysis(mexargs_in& in) {
     }
   }
 
-  workspace.add_expression(expr, mim, getfem::ga_workspace::dummy_region,
-                           der_order);
+  workspace.add_expression(expr, mim, getfem::dummy_mesh_region(), der_order);
   workspace.print(cout);
 }
 
@@ -597,9 +594,9 @@ gf_dirichlet(getfemint::mexargs_out& out,
 
 // To be parallelized
 void interpolate_or_extrapolate(mexargs_in &in, mexargs_out &out, int extrapolate) {
-  const getfem::mesh_fem *mf1 = in.pop().to_const_mesh_fem();
-  if (in.front().is_mesh_fem()) {
-    const getfem::mesh_fem *mf2 = in.pop().to_const_mesh_fem();
+  const getfem::mesh_fem *mf1 = to_meshfem_object(in.pop());
+  if (is_meshfem_object(in.front())) {
+    const getfem::mesh_fem *mf2 = to_meshfem_object(in.pop());
     gmm::row_matrix<getfem::model_real_sparse_vector>
       Maux(mf2->nb_dof(), mf1->nb_dof());
     getfem::interpolation(*mf1, *mf2, Maux, extrapolate);
@@ -636,18 +633,18 @@ void interpolate_or_extrapolate(mexargs_in &in, mexargs_out &out, int extrapolat
 }
 
 static const getfem::mesh_im *get_mim(mexargs_in &in) {
-  if (!in.front().is_mesh_im()) {
+  if (!is_meshim_object(in.front())) {
     THROW_BADARG("Since release 2.0 of getfem, all assembly functions"
                  " expect a mesh_im as their second argument");
   }
-  return in.pop().to_const_mesh_im();
+  return to_meshim_object(in.pop());
 }
 
 void assemble_source(size_type boundary_num,
                      mexargs_in &in, mexargs_out &out) {
   const getfem::mesh_im *mim = get_mim(in);
-  const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-  const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+  const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+  const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
   size_type region = boundary_num;
   
   unsigned q_dim = mf_u->get_qdim() / mf_d->get_qdim();
@@ -695,7 +692,7 @@ struct sub_gf_asm : virtual public dal::static_stored_object {
                    getfemint::mexargs_out& out) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_asm> psub_command;
+typedef std::shared_ptr<sub_gf_asm> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -706,7 +703,7 @@ template <typename T> static inline void dummy_func(T &) {}
                        getfemint::mexargs_out& out)                     \
       { dummy_func(in); dummy_func(out); code }                         \
     };                                                                  \
-    psub_command psubc = new subc;                                      \
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;         \
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;     \
     subc_tab[cmd_normalize(name)] = psubc;                              \
@@ -787,8 +784,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("mass matrix", 2, 4, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u1 = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_u2 = in.remaining() ? in.pop().to_const_mesh_fem() : mf_u1;
+       const getfem::mesh_fem *mf_u1 = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_u2 = in.remaining() ? to_meshfem_object(in.pop()) : mf_u1;
        size_type region = in.remaining() ? in.pop().to_integer():size_type(-1);
        getfem::mesh_region rg(region);
        mf_u1->linked_mesh().intersect_with_mpi_region(rg);
@@ -809,8 +806,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("laplacian", 4, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        darray A               = in.pop().to_darray(int(mf_d->nb_dof()));
        gf_real_sparse_by_col M(mf_u->nb_dof(), mf_u->nb_dof());
        size_type region = size_type(-1);
@@ -832,8 +829,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("linear elasticity", 5, 6, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        darray lambda          = in.pop().to_darray(int(mf_d->nb_dof()));
        darray mu              = in.pop().to_darray(int(mf_d->nb_dof()));
        gf_real_sparse_by_col M(mf_u->nb_dof(), mf_u->nb_dof());
@@ -884,15 +881,15 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("nonlinear elasticity", 3,-1,0,-1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
        darray U = in.pop().to_darray(int(mf_u->nb_dof()));
        std::string lawname = in.pop().to_string();
        /* a refaire , pas bon, le terme incompressible se passe de loi */
-       const getfem::abstract_hyperelastic_law &law
+       getfem::phyperelastic_law law
        = abstract_hyperelastic_law_from_name(lawname,
                                              mf_u->linked_mesh().dim());
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
-       darray param = in.pop().to_darray(int(law.nb_params()),
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
+       darray param = in.pop().to_darray(int(law->nb_params()),
                                          int(mf_d->nb_dof()));
 
        size_type region = size_type(-1);
@@ -905,14 +902,14 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
          if (cmd_strmatch(what, "tangent matrix")) {
            gf_real_sparse_by_col  K(mf_u->nb_dof(), mf_u->nb_dof());
            getfem::asm_nonlinear_elasticity_tangent_matrix(K, *mim, *mf_u, U,
-                                                           mf_d, param, law,rg);
+                                                           mf_d, param,*law,rg);
            out.pop().from_sparse(K);
          } else if (cmd_strmatch(what, "rhs")) {
            darray B = out.pop().create_darray_v(unsigned(mf_u->nb_dof()));
            getfem::asm_nonlinear_elasticity_rhs(B, *mim, *mf_u, U, mf_d,
-                                                param, law,rg);
+                                                param,*law, rg);
          } else if (cmd_strmatch(what, "incompressible tangent matrix")) {
-           const getfem::mesh_fem *mf_p = in.pop().to_const_mesh_fem();
+	   const getfem::mesh_fem *mf_p = to_meshfem_object(in.pop());
            darray P = in.pop().to_darray(int(mf_p->nb_dof()));
            gf_real_sparse_by_col  K(mf_u->nb_dof(), mf_u->nb_dof());
            gf_real_sparse_by_col  B(mf_u->nb_dof(), mf_p->nb_dof());
@@ -921,7 +918,7 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
            out.pop().from_sparse(K);
            out.pop().from_sparse(B);
          } else if (cmd_strmatch(what, "incompressible rhs")) {
-           const getfem::mesh_fem *mf_p = in.pop().to_const_mesh_fem();
+	   const getfem::mesh_fem *mf_p = to_meshfem_object(in.pop());
            darray P = in.pop().to_darray(int(mf_p->nb_dof()));
            darray RU = out.pop().create_darray_v(unsigned(mf_u->nb_dof()));
            darray RB = out.pop().create_darray_v(unsigned(mf_p->nb_dof()));
@@ -954,9 +951,9 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("stokes", 5, 6, 0, 2,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_p = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_p = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        darray           vec_d = in.pop().to_darray(int(mf_d->nb_dof()));
        gf_real_sparse_by_col  K(mf_u->nb_dof(), mf_u->nb_dof());
        gf_real_sparse_by_col  B(mf_u->nb_dof(), mf_p->nb_dof());
@@ -980,8 +977,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("helmholtz", 4, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        carray           wn = in.pop().to_carray(int(mf_d->nb_dof()));
        std::vector<complex_type> WN(wn.size());
        for (size_type i=0; i < wn.size(); ++i) WN[i] = gmm::sqr(wn[i]);
@@ -1005,8 +1002,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("bilaplacian", 4, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        darray           a = in.pop().to_darray(int(mf_d->nb_dof()));
        gf_real_sparse_by_col  A(mf_u->nb_dof(), mf_u->nb_dof());
        size_type region = size_type(-1);
@@ -1018,7 +1015,7 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        );
 
     /*@FUNC A = ('bilaplacian KL', @tmim mim, @tmf mf_u, @tmf mf_d, @dvec a, @dvec nu[, @int region])
-      Assembly of the matrix for the Bilaplacian problem with Kirchoff-Love formulation.
+      Assembly of the matrix for the Bilaplacian problem with Kirchhoff-Love formulation.
 
       :math:`\Delta(a(x)\Delta u) = 0`   with `a` scalar.
 
@@ -1027,8 +1024,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("bilaplacian KL", 5, 6, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        darray           a = in.pop().to_darray(int(mf_d->nb_dof()));
        darray           nu = in.pop().to_darray(int(mf_d->nb_dof()));
        gf_real_sparse_by_col  A(mf_u->nb_dof(), mf_u->nb_dof());
@@ -1072,7 +1069,7 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        );
 
 
-    /*@FUNC @CELL{HH, RR} = ('dirichlet', @int bnum, @tmim mim, @tmf mf_u, @tmf mf_d, @dmat H, @dvec R [, threshold])
+    /*@FUNC @CELL{HH, RR} = ('dirichlet', @int bnum, @tmim mim, @tmf mf_u, @tmf mf_d, @dmat H, @dvec R [, @scalar threshold])
     Assembly of Dirichlet conditions of type `h.u = r`.
 
     Handle `h.u = r` where h is a square matrix (of any rank) whose
@@ -1099,8 +1096,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       ("dirichlet", 6, 7, 2, 2,
        int boundary_num       = in.pop().to_integer();
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        mexarg_in in_h = in.pop();
        mexarg_in in_r = in.pop();
        double threshold = 1e-8;
@@ -1128,8 +1125,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       ("boundary qu term", 5, 5, 0, 1,
        int boundary_num       = in.pop().to_integer();
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_d = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_d = to_meshfem_object(in.pop());
        getfem::mesh_region rg(boundary_num);
        mf_u->linked_mesh().intersect_with_mpi_region(rg);
 
@@ -1186,6 +1183,56 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        getfem::ga_undefine_function(name);
        );
 
+    /*@FUNC ('define linear hardening function', @str name, @scalar sigma_y0, @scalar H, ... [@str 'Frobenius'])
+      Define a new linear hardening function under the name `name`, with
+      initial yield stress `sigma_y0` and hardening modulus H.
+      If an extra string argument with the value 'Frobenius' is provided,
+      the hardening function is expressed in terms of Frobenius norms of its
+      input strain and output stress, instead of their Von-Mises equivalents. @*/
+    sub_command
+      ("define linear hardening function", 3, 4, 0, 0,
+       std::string name = in.pop().to_string();
+       const double sigma_y0 = in.pop().to_scalar();
+       const double H = in.pop().to_scalar();
+       bool frobenius(false);
+       if (in.remaining()) frobenius = (in.pop().to_integer() != 0);
+       getfem::ga_define_linear_hardening_function(name, sigma_y0, H, frobenius);
+       );
+
+    /*@FUNC ('define Ramberg Osgood hardening function', @str name, @scalar sigma_ref, {@scalar eps_ref | @scalar E, @scalar alpha}, @scalar n[, @str 'Frobenius'])
+      Define a new Ramberg Osgood hardening function under the name `name`,
+      with initial yield stress `sigma_y0` and hardening modulus H.
+      If an extra string argument with the value 'Frobenius' is provided,
+      the hardening function is expressed in terms of Frobenius norms of its
+      input strain and output stress, instead of their Von-Mises equivalents. @*/
+    sub_command
+      ("define Ramberg Osgood hardening function", 4, 6, 0, 0,
+       const std::string name = in.pop().to_string();
+       const double sigma_ref = in.pop().to_scalar();
+       double eps_ref = in.pop().to_scalar();
+       double n = in.pop().to_scalar();
+
+       bool frobenius(false);
+       if (in.remaining()) {
+         mexarg_in argin = in.pop();
+         if (argin.is_string()) {
+           frobenius = cmd_strmatch(argin.to_string(), "frobenius");
+           if (in.remaining())
+             THROW_BADARG( "Wrong types of input arguments");
+         } else {
+           const double E(eps_ref);
+           const double alpha(n);
+           eps_ref = alpha*sigma_ref/E;
+           n = argin.to_scalar();
+           if (in.remaining())
+             frobenius = cmd_strmatch(in.pop().to_string(), "frobenius");
+         }
+       }
+
+       getfem::ga_define_Ramberg_Osgood_hardening_function
+         (name, sigma_ref, eps_ref, n, frobenius);
+       );
+
 
     /*@FUNC ('expression analysis', @str expression [, {@tm mesh | @tmim mim}] [, der_order] [, @tmodel model] [, @str varname, @int is_variable[, {@tmf mf | @tmimd mimd}], ...])
       Analyse a high-level generic assembly expression and print
@@ -1276,14 +1323,14 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       ("integral contact Uzawa projection", 9, 13, 0, 1,
        int boundary_num = in.pop().to_integer();
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
        // unsigned q_dim = mf_u->get_qdim();
        darray u = in.pop().to_darray();
        in.last_popped().check_trailing_dimension(int(mf_u->nb_dof()));
-       const getfem::mesh_fem *mf_lambda = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_lambda = to_meshfem_object(in.pop());
        darray vec_lambda = in.pop().to_darray();
        in.last_popped().check_trailing_dimension(int(mf_lambda->nb_dof()));
-       const getfem::mesh_fem *mf_obs = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_obs = to_meshfem_object(in.pop());
        darray obs = in.pop().to_darray();
        in.last_popped().check_trailing_dimension(int(mf_obs->nb_dof()));
        double r = in.pop().to_scalar();
@@ -1295,8 +1342,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
            const getfem::mesh_fem *mf_coeff = 0;
            darray vec_coeff;
            mexarg_in argin = in.pop();
-           if (argin.is_mesh_fem()) {
-             mf_coeff = argin.to_const_mesh_fem();
+           if (is_meshfem_object(argin)) {
+             mf_coeff = to_meshfem_object(argin);
              vec_coeff = in.pop().to_darray();
              in.last_popped().check_trailing_dimension(int(mf_coeff->nb_dof()));
            } else
@@ -1330,11 +1377,11 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       ("level set normal source term", 7, 7, 0, 1,
        int boundary_num = in.pop().to_integer();
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_lambda = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_u = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_lambda = to_meshfem_object(in.pop());
        darray vec_lambda = in.pop().to_darray();
        in.last_popped().check_trailing_dimension(int(mf_lambda->nb_dof()));
-       const getfem::mesh_fem *mf_obs = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_obs = to_meshfem_object(in.pop());
        darray obs = in.pop().to_darray();
        in.last_popped().check_trailing_dimension(int(mf_obs->nb_dof()));
        darray F = out.pop().create_darray_v(unsigned(mf_u->nb_dof()));
@@ -1353,9 +1400,9 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("lsneuman matrix", 4, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u1 = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_u2 = in.pop().to_const_mesh_fem();
-       getfem::level_set *ls1= in.pop().to_levelset();
+       const getfem::mesh_fem *mf_u1 = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_u2 = to_meshfem_object(in.pop());
+       getfem::level_set *ls1= to_levelset_object(in.pop());
        gf_real_sparse_by_col M(mf_u2->nb_dof(), mf_u1->nb_dof());
        size_type region = size_type(-1);
        if (in.remaining()) region = in.pop().to_integer();
@@ -1373,9 +1420,9 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("nlsgrad matrix", 4, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh_fem *mf_u1 = in.pop().to_const_mesh_fem();
-       const getfem::mesh_fem *mf_u2 = in.pop().to_const_mesh_fem();
-       getfem::level_set *ls1= in.pop().to_levelset();
+       const getfem::mesh_fem *mf_u1 = to_meshfem_object(in.pop());
+       const getfem::mesh_fem *mf_u2 = to_meshfem_object(in.pop());
+       getfem::level_set *ls1= to_levelset_object(in.pop());
        gf_real_sparse_by_col M(mf_u1->nb_dof(), mf_u2->nb_dof());
        size_type region = size_type(-1);
        if (in.remaining()) region = in.pop().to_integer();
@@ -1393,8 +1440,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("stabilization patch matrix", 5, 5, 0, 1,
        const getfem::mesh_im *mim = get_mim(in);
-       const getfem::mesh *mesh = in.pop().to_const_mesh();
-       const getfem::mesh_fem *mf_mult = in.pop().to_const_mesh_fem();
+       const getfem::mesh *mesh = to_mesh_object(in.pop());
+       const getfem::mesh_fem *mf_mult = to_meshfem_object(in.pop());
        double ratio_size= in.pop().to_scalar();
        double h= in.pop().to_scalar();
        gf_real_sparse_by_col M(mf_mult->nb_dof(), mf_mult->nb_dof());
