@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -43,7 +43,8 @@ iterative_gmm_solver(iterative_gmm_solver_type stype, gsparse &gsp,
 
   gprecond<T> id_precond;
   gprecond<T> *precond = &id_precond;
-  if (in.remaining()) precond = &in.pop().to_precond()->precond(T());
+  if (in.remaining())
+    precond = dynamic_cast<gprecond<T> *>(to_precond_object(in.pop()));
   precond->set_dimensions(gsp.nrows(), gsp.ncols());
 
   getfemint::interruptible_iteration iter(1e-16);
@@ -76,7 +77,7 @@ iterative_gmm_solver(iterative_gmm_solver_type stype, gsparse &gsp,
 
 void iterative_gmm_solver(iterative_gmm_solver_type stype,
                          getfemint::mexargs_in& in, getfemint::mexargs_out& out) {
-  dal::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
+  std::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
   gsparse &gsp = *pgsp;
   if (!gsp.is_complex() && in.front().is_complex())
     THROW_BADARG("please use a real right hand side, or convert the sparse matrix to a complex one");
@@ -120,22 +121,22 @@ struct sub_gf_linsolve : virtual public dal::static_stored_object {
                    getfemint::mexargs_out& out) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_linsolve> psub_command;
+typedef std::shared_ptr<sub_gf_linsolve> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
 
 #define sub_command(name, arginmin, arginmax, argoutmin, argoutmax, code) { \
-    struct subc : public sub_gf_linsolve {                                  \
-      virtual void run(getfemint::mexargs_in& in,                           \
-                       getfemint::mexargs_out& out)                         \
-      { dummy_func(in); dummy_func(out); code }                             \
-    };                                                                      \
-    psub_command psubc = new subc;                                          \
-    psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;             \
-    psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;         \
-    subc_tab[cmd_normalize(name)] = psubc;                                  \
-  }                           
+    struct subc : public sub_gf_linsolve {				\
+      virtual void run(getfemint::mexargs_in& in,			\
+                       getfemint::mexargs_out& out)			\
+      { dummy_func(in); dummy_func(out); code }				\
+    };									\
+    psub_command psubc = std::make_shared<subc>();			\
+    psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
+    psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
+    subc_tab[cmd_normalize(name)] = psubc;				\
+  }                        
 
 
 
@@ -181,7 +182,7 @@ void gf_linsolve(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       Alias for ::LINSOLVE('superlu',...)@*/
     sub_command
       ("lu", 2, 2, 0, 2,
-       dal::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
+       std::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
        gsparse &gsp = *pgsp;
        if (!gsp.is_complex() && in.front().is_complex())
          THROW_BADARG("please use a real right hand side, or convert the sparse matrix to a complex one");
@@ -196,7 +197,7 @@ void gf_linsolve(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     The condition number estimate `cond` is returned with the solution `U`.@*/
     sub_command
       ("superlu", 2, 2, 0, 2,
-       dal::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
+       std::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
        gsparse &gsp = *pgsp;
        if (!gsp.is_complex() && in.front().is_complex())
          THROW_BADARG("please use a real right hand side, or convert the sparse matrix to a complex one");
@@ -209,7 +210,7 @@ void gf_linsolve(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     Solve `M.U = b` using the MUMPS solver.@*/
     sub_command
       ("mumps", 2, 2, 0, 1,
-       dal::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
+       std::shared_ptr<gsparse> pgsp = in.pop().to_sparse();
        gsparse &gsp = *pgsp;
        if (!gsp.is_complex() && in.front().is_complex())
          THROW_BADARG("please use a real right hand side, or convert the sparse matrix to a complex one");
