@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2005-2015 Julien Pommier.
+ Copyright (C) 2005-2016 Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -18,10 +18,9 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_mesh.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id: gf_mesh.cc 5353 2016-07-26 13:36:19Z renard $
 #include <getfemint.h>
 #include <getfemint_workspace.h>
-#include <getfemint_mesh.h>
 #include <getfem/getfem_import.h>
 #include <getfem/getfem_regular_meshes.h>
 #include <getfem/getfem_mesher.h>
@@ -29,10 +28,10 @@
 using namespace getfemint;
 
 static void
-cartesian_mesh(getfem::mesh *pmesh, getfemint::mexargs_in &in, bool linear=true)
-{
+cartesian_mesh(getfem::mesh *pmesh, getfemint::mexargs_in &in,
+	       bool linear=true) {
   getfemint::size_type dim = in.remaining();
-
+  
   if (dim == 0) THROW_BADARG( "not enough input arguments");
 
   std::vector<darray> ppos(dim);
@@ -220,7 +219,7 @@ regular_simplices_mesh(getfem::mesh *pmesh, getfemint::mexargs_in &in) {
 static void
 curved_mesh(getfem::mesh *dest_mesh, getfemint::mexargs_in &in)
 {
-  const getfem::mesh *src_mesh = in.pop().to_const_mesh();
+  const getfem::mesh *src_mesh = extract_mesh_object(in.pop());
   darray F = in.pop().to_darray(src_mesh->points().index().last()+1);
 
   int dim = src_mesh->dim();
@@ -242,7 +241,7 @@ curved_mesh(getfem::mesh *dest_mesh, getfemint::mexargs_in &in)
 static void
 prismatic_mesh(getfem::mesh *dest_mesh, getfemint::mexargs_in &in)
 {
-  const getfem::mesh *src_mesh = in.pop().to_const_mesh();
+  const getfem::mesh *src_mesh = extract_mesh_object(in.pop());
   size_type nblay = in.pop().to_integer(1,2500000);
   short_type degree(1);
   if (in.remaining()) degree = short_type(in.pop().to_integer(1,2500000));
@@ -301,7 +300,7 @@ struct sub_gf_mesh : virtual public dal::static_stored_object {
 		   getfem::mesh *pmesh) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_mesh> psub_command;
+typedef std::shared_ptr<sub_gf_mesh> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -313,7 +312,7 @@ template <typename T> static inline void dummy_func(T &) {}
 		       getfem::mesh *pmesh)				\
       { dummy_func(in); dummy_func(out); code }				\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
@@ -488,7 +487,7 @@ void gf_mesh(getfemint::mexargs_in& m_in,
       Create a copy of a mesh.@*/
     sub_command
       ("clone", 1, 1, 0, 1,
-       const getfem::mesh *m2 = in.pop().to_const_mesh();
+       const getfem::mesh *m2 = extract_mesh_object(in.pop());
        pmesh->copy_from(*m2);
        );
 
@@ -512,8 +511,7 @@ void gf_mesh(getfemint::mexargs_in& m_in,
     sub_command
       ("generate", 2, 4, 0, 1,
 
-       const getfem::mesher_signed_distance *pmo
-            = in.pop().to_const_mesher_object();
+       getfem::pmesher_signed_distance psd = to_mesher_object(in.pop());
        double h = in.pop().to_scalar();
        int K = 1;
        if (in.remaining()) K = in.pop().to_integer(1,6);
@@ -529,18 +527,14 @@ void gf_mesh(getfemint::mexargs_in& m_in,
        int prefind = 1;
        int max_iter = 400;
 
-       getfem::build_mesh(*pmesh, *pmo, h, fixed, K, -1, max_iter, prefind);
+       getfem::build_mesh(*pmesh, psd, h, fixed, K, -1, max_iter, prefind);
        );
-
   }
 
 
   if (m_in.narg() < 1)  THROW_BADARG( "Wrong number of input arguments");
 
-  getfemint_mesh *mi_mesh =
-    getfemint_mesh::get_from(new getfem::mesh);
-  m_out.pop().from_object_id(mi_mesh->get_id(), MESH_CLASS_ID);
-  getfem::mesh *pmesh = &mi_mesh->mesh();
+  auto mesh = std::make_shared<getfem::mesh>();
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
@@ -550,7 +544,9 @@ void gf_mesh(getfemint::mexargs_in& m_in,
     check_cmd(cmd, it->first.c_str(), m_in, m_out, it->second->arg_in_min,
 	      it->second->arg_in_max, it->second->arg_out_min,
 	      it->second->arg_out_max);
-    it->second->run(m_in, m_out, pmesh);
+    it->second->run(m_in, m_out, mesh.get());
+
+    m_out.pop().from_object_id(store_mesh_object(mesh), MESH_CLASS_ID);
   }
   else bad_cmd(init_cmd);
 
