@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2006-2012 Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2006-2016 Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,19 +16,18 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
-// $Id: gf_mesh_fem_get.cc 4705 2014-07-09 07:31:24Z logari81 $
-#include <getfemint_misc.h>
-#include <getfemint_mesh_fem.h>
-#include <getfemint_integ.h>
-#include <getfemint_pfem.h>
+// $Id: gf_mesh_fem_get.cc 5353 2016-07-26 13:36:19Z renard $
+
+#include <getfem/getfem_fem.h>
 #include <getfem/getfem_export.h>
 #include <getfem/getfem_mesh_fem_level_set.h>
-#include <getfemint_mesh_levelset.h>
 #include <getfem/getfem_mesh_im.h>
 #include <getfem/getfem_partial_mesh_fem.h>
+#include <getfemint_misc.h>
 #include <getfemint_gsparse.h>
+#include <getfemint_workspace.h>
 
 using namespace getfemint;
 
@@ -70,9 +69,8 @@ static void get_basic_fem_of_convexes(const getfem::mesh_fem& mf,
   std::vector<id_type> ids; ids.reserve(cvlst.card());
   for (dal::bv_visitor cv(cvlst); !cv.finished(); ++cv) {
     if (mf.convex_index().is_in(cv)) {
-      getfemint_pfem *gfi_pf =
-        getfemint_pfem::get_from(mf.fem_of_element(cv));
-      ids.push_back(gfi_pf->get_id());
+      id_type id = store_fem_object(mf.fem_of_element(cv));
+      ids.push_back(id);
     } else
       ids.push_back(id_type(-1));
   }
@@ -118,7 +116,9 @@ non_conformal_dof(getfem::mesh_fem &mf, mexargs_in &in, mexargs_out &out) {
   else cvlst = m.convex_index();
 
   for (dal::bv_visitor ic(cvlst); !ic.finished(); ++ic) {
-    check_cv_fem(mf, ic);
+    if (!mf.convex_index()[ic])
+      THROW_ERROR( "convex " << ic+config::base_index() << " has no FEM");
+
     for (short_type f = 0; f < m.structure_of_convex(ic)->nb_faces(); f++) {
       bgeot::short_type q;
       if (!m.is_convex_having_neighbour(ic, f)) {
@@ -213,11 +213,10 @@ struct sub_gf_mf_get : virtual public dal::static_stored_object {
   int arg_in_min, arg_in_max, arg_out_min, arg_out_max;
   virtual void run(getfemint::mexargs_in& in,
 		   getfemint::mexargs_out& out,
-		   getfemint_mesh_fem *mi_mf,
 		   getfem::mesh_fem *mf) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_mf_get> psub_command;
+typedef std::shared_ptr<sub_gf_mf_get> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -226,12 +225,10 @@ template <typename T> static inline void dummy_func(T &) {}
     struct subc : public sub_gf_mf_get {				\
       virtual void run(getfemint::mexargs_in& in,			\
 		       getfemint::mexargs_out& out,			\
-		       getfemint_mesh_fem *mi_mf,			\
 		       getfem::mesh_fem *mf)				\
-      { dummy_func(in); dummy_func(out); dummy_func(mi_mf);		\
-	dummy_func(mf); code }						\
+      { dummy_func(in); dummy_func(out); dummy_func(mf); code }		\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
@@ -511,6 +508,40 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        out.pop().from_sparse(M);
        );
 
+    /*@GET Vr = ('reduce vector', @vec V)
+    Multiply the provided vector V with the extension matrix of the @tmf.@*/
+    sub_command
+      ("reduce vector", 1, 1, 0, 1,
+       if (in.front().is_complex()) {
+         carray V = in.pop().to_carray(-1);
+         std::vector<std::complex<double> > Vr(mf->nb_dof());
+         mf->reduce_vector(V, Vr);
+         out.pop().from_dcvector(Vr);
+       } else {
+         darray V = in.pop().to_darray(-1);
+         std::vector<double> Vr(mf->nb_dof());
+         mf->reduce_vector(V, Vr);
+         out.pop().from_dcvector(Vr);
+       }
+       );
+
+    /*@GET Ve = ('extend vector', @vec V)
+    Multiply the provided vector V with the reduction matrix of the @tmf.@*/
+    sub_command
+      ("extend vector", 1, 1, 0, 1,
+       if (in.front().is_complex()) {
+         carray V = in.pop().to_carray(-1);
+         std::vector<std::complex<double> > Ve(mf->nb_basic_dof());
+         mf->extend_vector(V, Ve);
+         out.pop().from_dcvector(Ve);
+       } else {
+         darray V = in.pop().to_darray(-1);
+         std::vector<double> Ve(mf->nb_basic_dof());
+         mf->extend_vector(V, Ve);
+         out.pop().from_dcvector(Ve);
+       }
+       );
+
 
     /*@GET DOFs = ('basic dof on region',@mat Rs)
     Return the list of basic dof (before the optional reduction) lying on one
@@ -675,7 +706,17 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
       Return a reference to the @tmesh object linked to `mf`.@*/
     sub_command
       ("linked mesh", 0, 0, 0, 1,
-       out.pop().from_object_id(mi_mf->linked_mesh_id(), MESH_CLASS_ID);
+       id_type id =  workspace().object((const void *)(&mf->linked_mesh()));
+       if (id == id_type(-1)) {
+	 auto pst = workspace().hidden_object(workspace().object(mf),
+					      &mf->linked_mesh());
+	 if (!pst.get()) THROW_INTERNAL_ERROR;
+	 std::shared_ptr<getfem::mesh> pm = 
+	   std::const_pointer_cast<getfem::mesh>
+	   (std::dynamic_pointer_cast<const getfem::mesh>(pst));
+	 id = store_mesh_object(pm);
+       }
+       out.pop().from_object_id(id, MESH_CLASS_ID);
        );
 
 
@@ -684,7 +725,9 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
       (identical to MESH:GET('linked mesh'))@*/
     sub_command
       ("mesh", 0, 0, 0, 1,
-       out.pop().from_object_id(mi_mf->linked_mesh_id(), MESH_CLASS_ID);
+       id_type id =  workspace().object((const void *)(&mf->linked_mesh()));
+       if (id == id_type(-1)) THROW_INTERNAL_ERROR;
+       out.pop().from_object_id(id, MESH_CLASS_ID);
        );
 
 
@@ -712,8 +755,8 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        int count = 1;
        while (in.remaining()) {
 	 const getfem::mesh_fem *mf2 = mf;
-	 if (in.remaining() >= 2 && in.front().is_mesh_fem())
-	   mf2 = in.pop().to_const_mesh_fem();
+	 if (in.remaining() >= 2 && is_meshfem_object(in.front()))
+	   mf2 = to_meshfem_object(in.pop());
 	 darray U = in.pop().to_darray();
 	 in.last_popped().check_trailing_dimension(int(mf2->nb_dof()));
 	 exp.write_point_data(*mf2, U, get_vtk_dataset_name(in, count));
@@ -758,8 +801,8 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        if (edges) exp.exporting_mesh_edges();
        while (in.remaining()) {
 	 const getfem::mesh_fem *mf2 = mf;
-	 if (in.remaining() >= 2 && in.front().is_mesh_fem())
-	   mf2 = in.pop().to_const_mesh_fem();
+	 if (in.remaining() >= 2 && is_meshfem_object(in.front()))
+	   mf2 = to_meshfem_object(in.pop());
 	 darray U = in.pop().to_darray();
 	 in.last_popped().check_trailing_dimension(int(mf2->nb_dof()));
 	 exp.write_point_data(*mf2, U, get_dx_dataset_name(in));
@@ -785,8 +828,8 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        exp.write(*mf,name);
        while (in.remaining()) {
 	 const getfem::mesh_fem *mf2 = mf;
-	 if (in.remaining() >= 2 && in.front().is_mesh_fem()) {
-	   mf2 = in.pop().to_const_mesh_fem();
+	 if (in.remaining() >= 2 && is_meshfem_object(in.front())) {
+	   mf2 = to_meshfem_object(in.pop());
 	 }
 	 darray U = in.pop().to_darray();
 	 in.last_popped().check_trailing_dimension(int(mf2->nb_dof()));
@@ -812,7 +855,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
     the convexes which are not crosses by the levelset!@*/
     sub_command
       ("dof_from_im", 1, 2, 0, 1,
-       const getfem::mesh_im &mim = *in.pop().to_const_mesh_im();
+       const getfem::mesh_im &mim = *to_meshim_object(in.pop());
        int P = -1;
        if (&mim.linked_mesh() != &mf->linked_mesh())
 	 THROW_BADARG("the mesh_im uses a different mesh");
@@ -857,7 +900,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
     sub_command
       ("has_linked_mesh_levelset", 0, 0, 0, 1,
        getfem::mesh_fem_level_set *mfls =
-       dynamic_cast<getfem::mesh_fem_level_set*>(mf);
+       dynamic_cast<getfem::mesh_fem_level_set *>(mf);
        out.pop().from_integer(mfls ? 1 : 0);
        );
 
@@ -871,11 +914,9 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        if (mfls) {
 	 getfem::mesh_level_set *mls =
 	   const_cast<getfem::mesh_level_set*>(&mfls->linked_mesh_level_set());
-	 getfemint_mesh_levelset *gfi_mls =
-	   getfemint_mesh_levelset::get_from(mls);
-	 assert(gfi_mls);
-	 out.pop().from_object_id(gfi_mls->get_id(),
-				  MESH_LEVELSET_CLASS_ID);
+	 id_type id = workspace().object((const void *)(mls));
+	 GMM_ASSERT1(id != id_type(-1), "Unknown mesh_level_set !");
+	 out.pop().from_object_id(id, MESH_LEVELSET_CLASS_ID);
        } else THROW_BADARG("not a mesh_fem using a mesh_levelset");
        );
 
@@ -884,8 +925,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
 
 
   if (m_in.narg() < 2)  THROW_BADARG( "Wrong number of input arguments");
-  getfemint_mesh_fem *mi_mf = m_in.pop().to_getfemint_mesh_fem();
-  getfem::mesh_fem *mf   = &mi_mf->mesh_fem();
+  getfem::mesh_fem *mf   = to_meshfem_object(m_in.pop());
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
@@ -895,7 +935,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
     check_cmd(cmd, it->first.c_str(), m_in, m_out, it->second->arg_in_min,
 	      it->second->arg_in_max, it->second->arg_out_min,
 	      it->second->arg_out_max);
-    it->second->run(m_in, m_out, mi_mf, mf);
+    it->second->run(m_in, m_out, mf);
   }
   else bad_cmd(init_cmd);
 
