@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 1999-2015 Yves Renard
+ Copyright (C) 1999-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -39,29 +39,21 @@
 #define GETFEM_MESH_H__
 
 #include <bitset>
-#include "dal_shared_ptr.h"
 #include "bgeot_ftool.h"
 #include "bgeot_mesh.h"
 #include "bgeot_geotrans_inv.h"
 #include "getfem_context.h"
 #include "getfem_mesh_region.h"
-
-#if defined(__GNUC__) 
-#if __cplusplus > 199711L
-#include <memory>  //on GCC need to check for C++ 11
-#endif
-#else
-#include <memory>  //all the others like Intel and MSVC have it
-#endif
+#include <memory>
 
 
 namespace getfem {
 
   /* Version counter for convexes. */
-  gmm::uint64_type APIDECL act_counter(void);
+  gmm::uint64_type APIDECL act_counter();
 
   class integration_method;
-  typedef boost::intrusive_ptr<const integration_method> pintegration_method;
+  typedef std::shared_ptr<const integration_method> pintegration_method;
 
   /**@addtogroup mesh*/
   /**@{*/
@@ -100,19 +92,10 @@ namespace getfem {
       (same thing for the convexes, always use convex_index()).
   */
 
-  class APIDECL mesh : virtual public dal::static_stored_object,
-                       public bgeot::basic_mesh,
-                       public context_dependencies 
-
-//allowing mesh::shared_from_this() call
-#if defined(__GNUC__) 
-#if __cplusplus > 199711L
-           , public std::enable_shared_from_this<mesh>
-#endif
-#else
-           , public std::enable_shared_from_this<mesh>
-#endif
-  
+  class APIDECL mesh : public bgeot::basic_mesh,
+                       public context_dependencies,
+                       virtual public dal::static_stored_object,
+                       public std::enable_shared_from_this<mesh>
   {
   public :
 
@@ -139,7 +122,7 @@ namespace getfem {
     mutable bool cuthill_mckee_uptodate;
     dal::dynamic_array<gmm::uint64_type> cvs_v_num;
     mutable std::vector<size_type> cmk_order; // cuthill-mckee
-    void init(void);
+    void init();
 
 #if GETFEM_PARA_LEVEL > 1
     mutable bool modified;
@@ -148,16 +131,16 @@ namespace getfem {
     // mutable dal::dynamic_array<mesh_region> mpi_sub_region;
     mutable dal::bit_vector valid_sub_regions;
 
-    void touch(void) {
+    void touch() {
       modified = true; cuthill_mckee_uptodate = false;
       context_dependencies::touch();
     }
-    void compute_mpi_region(void) const ;
+    void compute_mpi_region() const ;
     void compute_mpi_sub_region(size_type) const;
 
   public :
 
-    const mesh_region& get_mpi_region(void) const
+    const mesh_region& get_mpi_region() const
     { if (modified) compute_mpi_region(); return mpi_region; }
     const mesh_region& get_mpi_sub_region(size_type n) const {
       if (modified) compute_mpi_region();
@@ -167,10 +150,10 @@ namespace getfem {
     }
     void intersect_with_mpi_region(mesh_region &rg) const;
 #else
-    void touch(void)
+    void touch()
     { cuthill_mckee_uptodate = false; context_dependencies::touch(); }
   public :
-    const mesh_region get_mpi_region(void) const
+    const mesh_region get_mpi_region() const
     { return mesh_region::all_convexes(); }
     const mesh_region get_mpi_sub_region(size_type n) const {
       if (n == size_type(-1)) return get_mpi_region();
@@ -182,14 +165,17 @@ namespace getfem {
     /// Constructor.
     explicit mesh(const std::string name = "");
     mesh(const bgeot::basic_mesh &m, const std::string name = "");
+    mesh(const mesh &m);
+    mesh &operator=(const mesh &m);
+
     inline std::string get_name() const {return name_;}
-    void update_from_context(void) const {}
+    void update_from_context() const {}
     /// Mesh dimension.
-    dim_type dim(void) const { return pts.dim(); }
+    dim_type dim() const { return pts.dim(); }
     /// Return the array of PT.
-    const PT_TAB &points(void) const { return pts; }
+    const PT_TAB &points() const { return pts; }
     /// Return the array of PT.
-    PT_TAB &points(void) { return pts; }
+    PT_TAB &points() { return pts; }
 
     /// Return a (pseudo)container of the points of a given convex
     ref_mesh_pt_ct points_of_convex(size_type ic) const {
@@ -211,18 +197,22 @@ namespace getfem {
     /** Add the point pt to the mesh and return the index of the
         point.
 
-        If the point is too close to an existing point, the
-        function does not create a new point, and returns the index of the
+        If the point is too close to an existing point and remove_duplicated_nodes = true,
+        the function does not create a new point, and returns the index of the
         already existing point.
         @param pt the point coordinates.
     */
-    size_type add_point(const base_node &pt) { return pts.add_node(pt); }
+    size_type add_point(const base_node &pt,
+                        const scalar_type tol=scalar_type(0),
+                        bool remove_duplicated_nodes = true) {
+      return pts.add_node(pt, tol, remove_duplicated_nodes);
+    }
     //                        scalar_type characteristic_size = scalar_type(1));
 
     /// Give the number of geometrical nodes in the mesh.
-    size_type nb_points(void) const { return pts.card(); }
+    size_type nb_points() const { return pts.card(); }
     /// Return the points index
-    const dal::bit_vector &points_index(void) const { return pts.index(); }
+    const dal::bit_vector &points_index() const { return pts.index(); }
     /** Delete the point of index i from the mesh if it is not linked to a
         convex.
     */
@@ -278,7 +268,8 @@ namespace getfem {
         @return the number of the new convex.
     */
     template<class ITER>
-    size_type add_convex_by_points(bgeot::pgeometric_trans pgt, ITER ipts);
+    size_type add_convex_by_points(bgeot::pgeometric_trans pgt, ITER ipts,
+                                   const scalar_type tol=scalar_type(0));
 
     /** Add a simplex to the mesh, given its dimension and point numbers.
      */
@@ -426,7 +417,7 @@ namespace getfem {
     */
     scalar_type convex_area_estimate(size_type ic, size_type degree=2) const;
     /** Return an estimate of the convex largest dimension. @see getfem::convex_quality_estimate */
-    scalar_type convex_radius_estimate(size_type ic) const;
+    virtual scalar_type convex_radius_estimate(size_type ic) const;
     /** Return an estimate of the convex smallest dimension. @see getfem::convex_radius_estimate */
     scalar_type minimal_convex_radius_estimate() const;
     /** Return an estimate of the convex largest dimension. @see getfem::convex_radius_estimate */
@@ -477,11 +468,11 @@ namespace getfem {
     void sup_convex_from_regions(size_type cv);
     /** Pack the mesh : renumber convexes and nodes such that there
         is no holes in their numbering. Do NOT do the Cuthill-McKee. */
-    void optimize_structure(void);
+    void optimize_structure();
     /// Return the list of convex IDs for a Cuthill-McKee ordering
     const std::vector<size_type> &cuthill_mckee_ordering() const;
     /// Erase the mesh.
-    void clear(void);
+    void clear();
     /** Write the mesh to a file. The format is getfem-specific.
         @param name the file name.
     */
@@ -502,10 +493,7 @@ namespace getfem {
     void read_from_file(std::istream &ist);
     /** Clone a mesh */
     void copy_from(const mesh& m); /* might be the copy constructor */
-    size_type memsize(void) const;
-    ~mesh() {
-      if (Bank_info) delete Bank_info;
-    }
+    size_type memsize() const;
 
     friend class mesh_region;
   private:
@@ -542,7 +530,7 @@ namespace getfem {
       edge_set edges;
     };
 
-    Bank_info_struct *Bank_info;
+    std::unique_ptr<Bank_info_struct> Bank_info;
 
     std::string name_; //optional name of the mesh
     void set_name(const std::string&);
@@ -567,6 +555,10 @@ namespace getfem {
 
   };
 
+  /* Dummy mesh for default parameter of functions. */
+  const mesh &dummy_mesh();
+
+
   inline void APIDECL mesh::add_face_to_set(size_type s, size_type c, short_type f) {
     region(s).add(c,f);
   }
@@ -579,11 +571,11 @@ namespace getfem {
 
  template<class ITER>
     size_type mesh::add_convex_by_points(bgeot::pgeometric_trans pgt,
-                                                                   ITER ipts)
+                                         ITER ipts, const scalar_type tol)
   {
     short_type nb = short_type(pgt->nb_points());
     std::vector<size_type> ind(nb);
-    for (short_type i = 0; i < nb; ++ipts, ++i) ind[i] = add_point(*ipts);
+    for (short_type i = 0; i < nb; ++ipts, ++i) ind[i] = add_point(*ipts, tol);
     return add_convex(pgt, ind.begin());
   }
 
@@ -618,18 +610,18 @@ namespace getfem {
       of the convex area.
    */
   scalar_type APIDECL convex_area_estimate(bgeot::pgeometric_trans pgt,
-                                   const base_matrix& pts,
-                                   pintegration_method pim);
+                                           const base_matrix& pts,
+                                           pintegration_method pim);
 
   /** rough estimate of the maximum value of the condition
    * number of the jacobian of the geometric transformation */
   scalar_type APIDECL convex_quality_estimate(bgeot::pgeometric_trans pgt,
-                                      const base_matrix& pts);
+                                              const base_matrix& pts);
 
   /** rough estimate of the radius of the convex using the largest eigenvalue
    * of the jacobian of the geometric transformation */
   scalar_type APIDECL convex_radius_estimate(bgeot::pgeometric_trans pgt,
-                                     const base_matrix& pts);
+                                             const base_matrix& pts);
 
   /* stores a convex face. if f == -1, it is the whole convex.             */
   struct convex_face;
@@ -670,6 +662,13 @@ namespace getfem {
   inline mesh_region APIDECL outer_faces_of_mesh(const mesh &m)
   { mesh_region fl; outer_faces_of_mesh(m,m.convex_index(),fl); return fl; }
 
+  /** Select all the faces sharing at least two element of the given mesh
+      region. Each face is represented only once and is arbitrarily chosen
+      between the two neighbour elements.
+   */
+  mesh_region APIDECL inner_faces_of_mesh
+  (const mesh &m, mesh_region mr = mesh_region::all_convexes());
+  
   /** Select in the region mr the faces of the mesh m with their unit
       outward vector having a maximal angle "angle" with the vector V.
    */
