@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2005-2015 Julien Pommier
+ Copyright (C) 2005-2016 Julien Pommier
 
  This file is a part of GetFEM++
 
@@ -35,7 +35,7 @@ namespace getfem {
     return os;
   }
 
-  std::ostream &operator<<(std::ostream &os, const mesh_level_set::zoneset &zs) {
+  std::ostream &operator<<(std::ostream &os,const mesh_level_set::zoneset &zs) {
     os << "zoneset[";
     for (mesh_level_set::zoneset::const_iterator it = zs.begin(); 
 	 it != zs.end(); ++it) {
@@ -100,9 +100,10 @@ struct Chrono {
 
   mesh_level_set::~mesh_level_set() {}
 
-  static void interpolate_face(mesh &m, dal::bit_vector& ptdone, 
+  static void interpolate_face(const bgeot::pgeometric_trans &pgt,
+			       mesh &m, dal::bit_vector& ptdone, 
 			       const std::vector<size_type>& ipts,
-			       bgeot::pconvex_structure cvs, 
+			       const bgeot::pconvex_structure &cvs, 
 			       size_type nb_vertices,
 			       const std::vector<dal::bit_vector> &constraints,
 			       const std::vector<const
@@ -114,27 +115,33 @@ struct Chrono {
 	fpts.resize(cvs->nb_points_of_face(f));
 	for (size_type k=0; k < fpts.size(); ++k)
 	  fpts[k] = ipts[cvs->ind_points_of_face(f)[k]];
-	interpolate_face(m,ptdone,fpts,cvs->faces_structure()[f], nb_vertices,
-			 constraints, list_constraints);
+	interpolate_face(pgt, m,ptdone,fpts,cvs->faces_structure()[f],
+			 nb_vertices, constraints, list_constraints);
       }
     }
 
-    // cout << "ipts.size() = " << ipts.size() << endl;
-    // cout << " nb_vertices = " <<  nb_vertices << endl;
+    if (noisy) {
+      cout << "ipts.size() = " << ipts.size() << endl;
+      cout << " nb_vertices = " <<  nb_vertices << endl;
+    }
 
     dal::bit_vector cts; size_type cnt = 0;
     for (size_type i=0; i < ipts.size(); ++i) {
       // cout << "ipts[i] = " << ipts[i] << endl;
       if (ipts[i] < nb_vertices) {
-	// cout << "point " << i << " constraints[ipts[i]] = " << constraints[ipts[i]] << endl;
+	if (noisy)
+	  cout << "point " << i << " coordinates "
+	       << m.points()[ipts[i]] << " constraints[ipts[i]] = "
+	       << constraints[ipts[i]] << endl;
 	if (cnt == 0) cts = constraints[ipts[i]];
 	else cts &= constraints[ipts[i]];
 	++cnt;
       }
     }
 
+    if (noisy) cout << "cts = " << cts << endl;
+      
     if (cts.card()) {
-      // cout << "cts = " << cts << endl;
       // dal::bit_vector new_cts;
       for (size_type i=0; i < ipts.size(); ++i) {
 	if (ipts[i] >= nb_vertices && !ptdone[ipts[i]]) {
@@ -142,10 +149,14 @@ struct Chrono {
 	  // if (cts.card() > 1)
 	  //   cout << "WARNING, projection sur " << cts << endl;
 	  if (!pure_multi_constraint_projection(list_constraints, P, cts)) {
-	    GMM_WARNING1("Pure multi has failed in interpolate_face !! Original point " << m.points()[ipts[i]] << " projection " << P);
-	    // GMM_ASSERT1(false, "");
+	    GMM_WARNING1("Pure multi has failed in interpolate_face !! "
+			 "Original point " << m.points()[ipts[i]]
+			 << " projection " << P);
 	  } else {
-	    m.points()[ipts[i]] = P;
+	    if (pgt->convex_ref()->is_in(P) > 1E-8) {
+	      GMM_WARNING1("Projected point outside the reference convex ! "
+			   "Projection canceled. P = " << P);
+	    } else m.points()[ipts[i]] = P;
 	  }
 	  ptdone[ipts[i]] = true;
 	  // dist(P, new_cts);
@@ -256,13 +267,13 @@ struct Chrono {
   };
 
 
-  static mesher_signed_distance *new_ref_element(bgeot::pgeometric_trans pgt) {
+  static pmesher_signed_distance new_ref_element(bgeot::pgeometric_trans pgt) {
     dim_type n = pgt->structure()->dim();
     size_type nbp = pgt->basic_structure()->nb_points();
     /* Identifying simplexes.                                          */
     if (nbp == size_type(n+1) &&
 	pgt->basic_structure() == bgeot::simplex_structure(n)) {
-	return new mesher_simplex_ref(n);
+	return new_mesher_simplex_ref(n);
     }
     
     /* Identifying parallelepiped.                                     */
@@ -270,13 +281,13 @@ struct Chrono {
 	pgt->basic_structure() == bgeot::parallelepiped_structure(n)) {
       base_node rmin(n), rmax(n);
       std::fill(rmax.begin(), rmax.end(), scalar_type(1));
-      return new mesher_rectangle(rmin, rmax);
+      return new_mesher_rectangle(rmin, rmax);
     }
     
     /* Identifying prisms.                                             */
     if (nbp == size_type(2 * n) &&
 	pgt->basic_structure() == bgeot::prism_structure(n)) {
-      return new mesher_prism_ref(n);
+      return new_mesher_prism_ref(n);
     }
     
     GMM_ASSERT1(false, "This element is not taken into account. Contact us");
@@ -382,11 +393,11 @@ struct Chrono {
 				   scalar_type radius_cv) {
     
     cut_cv[cv] = convex_info();
-    cut_cv[cv].pmsh = pmesh(new mesh);
+    cut_cv[cv].pmsh = std::make_shared<mesh>();
     if (noisy) cout << "cutting element " << cv << endl;
     bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
-    std::unique_ptr<mesher_signed_distance> ref_element(new_ref_element(pgt));
-    std::vector<mesher_level_set> mesher_level_sets;
+    pmesher_signed_distance ref_element = new_ref_element(pgt);
+    std::vector<pmesher_signed_distance> mesher_level_sets;
     
     size_type n = pgt->structure()->dim();
     size_type nbtotls = primary.card() + secondary.card();
@@ -411,16 +422,16 @@ struct Chrono {
 	base_node X(n); gmm::fill_random(X);
 	K = std::max(K, (level_sets[ll])->degree());
 	mesher_level_sets.push_back(level_sets[ll]->mls_of_convex(cv, 0));
-	mesher_level_set &mls(mesher_level_sets.back());
-	list_constraints.push_back(&mesher_level_sets.back());
-	r0 = std::min(r0, curvature_radius_estimate(mls, X, true));
+	pmesher_signed_distance mls(mesher_level_sets.back());
+	list_constraints.push_back(mesher_level_sets.back().get());
+	r0 = std::min(r0, curvature_radius_estimate(*mls, X, true));
 	GMM_ASSERT1(gmm::abs(r0) >= 1e-13, "Something wrong in your level "
 		    "set ... curvature radius = " << r0);
 	if (secondary[ll]) {
 	  mesher_level_sets.push_back(level_sets[ll]->mls_of_convex(cv, 1));
-	  mesher_level_set &mls2(mesher_level_sets.back());
-	  list_constraints.push_back(&mesher_level_sets.back());
-	  r0 = std::min(r0, curvature_radius_estimate(mls2, X, true));
+	  pmesher_signed_distance mls2(mesher_level_sets.back());
+	  list_constraints.push_back(mesher_level_sets.back().get());
+	  r0 = std::min(r0, curvature_radius_estimate(*mls2, X, true));
 	}
       }
     }
@@ -505,7 +516,7 @@ struct Chrono {
 
       dmin = 2.*h0;
       if (noisy) cout << "dmin = " << dmin << " h0 = " << h0 << endl;
-      if (noisy) cout << "cetait le convexe " << cv << endl;
+      if (noisy) cout << "convex " << cv << endl;
       
       dal::bit_vector retained_points
 	= mesh_points.decimate(*ref_element, dmin);
@@ -543,7 +554,7 @@ struct Chrono {
 	msh.clear(); 
 	
 	for (size_type i = 0; i <  fixed_points.size(); ++i) {
-	  size_type j = msh.add_point(fixed_points[i]); // remettre le add_norepeat ?
+	  size_type j = msh.add_point(fixed_points[i]);
 	  assert(j == i);
 	}
 	
@@ -552,7 +563,7 @@ struct Chrono {
 	  size_type j = msh.add_convex(bgeot::simplex_geotrans(n,1),
 				       gmm::vect_const_begin(gmm::mat_col(simplexes, i)));
 	  /* remove flat convexes => beware the final mesh may not be conformal ! */
-	  if (msh.convex_quality_estimate(j) < 1E-12) msh.sup_convex(j);
+	  if (msh.convex_quality_estimate(j) < 1E-10) msh.sup_convex(j);
 	  else {
 	    std::vector<scalar_type> signs(list_constraints.size());
 	    std::vector<size_type> prev_point(list_constraints.size());
@@ -562,7 +573,7 @@ struct Chrono {
 		  (*(list_constraints[jj]))(msh.points_of_convex(j)[ii]);
 		if (gmm::abs(dd) > radius_cv * 1E-7) {
 		  if (dd * signs[jj] < 0.0) {
-		    if (noisy) cout << "Intersection trouvee ... " << jj
+		    if (noisy) cout << "Intersection found ... " << jj
 				    << " dd = " << dd << " convex quality = "
 				    << msh.convex_quality_estimate(j) << "\n";
 		    // calcul d'intersection
@@ -614,6 +625,7 @@ struct Chrono {
       }
       if (!h0_is_ok) continue;
 
+      // Produces an order K mesh for K > 1 (with affine element for the moment)
       if (K>1) {
 	for (dal::bv_visitor_c j(msh.convex_index()); !j.finished(); ++j) {
 	  bgeot::pgeometric_trans pgt2 = bgeot::simplex_geotrans(n, K);
@@ -623,8 +635,8 @@ struct Chrono {
 	      (pgt2->convex_ref()->points()[k], msh.points_of_convex(j));
 	  }
 	  msh.sup_convex(j);
-	  /* some new point will be added to the mesh, but the initial ones (those on the
-	     convex vertices) are kepts */
+	  /* some new point will be added to the mesh, but the initial ones
+	     (those on the convex vertices) are kepts */
 	  size_type jj = msh.add_convex_by_points(pgt2, cvpts.begin());
 	  assert(jj == j);
 	}
@@ -653,7 +665,7 @@ struct Chrono {
 	      assert(K>1);
 	      continue; /* ignore additional point inserted when K>1 */
 	    }
-	    if (first) { cts = fixed_points_constraints[fpts[k]]; first = false; }
+	    if (first) { cts = fixed_points_constraints[fpts[k]]; first=false; }
 	    else cts &= fixed_points_constraints[fpts[k]];
 	  }
 	  for (dal::bv_visitor ii(cts); !ii.finished(); ++ii) {
@@ -664,9 +676,12 @@ struct Chrono {
       }
 
       
-      if (K > 1) { //  ne faire que sur les convexes concerns ..
+      if (K > 1) { // To be done only on the level-set ...
 	dal::bit_vector ptdone;
 	std::vector<size_type> ipts;
+	// for (mr_visitor icv(ls_border_faces); !icv.finished(); ++icv) {
+	//  size_type i = icv.cv();
+	//   short_type f = icv.f();
 	for (dal::bv_visitor i(msh.convex_index()); !i.finished(); ++i) {
 	  for (short_type f = 0; f <= n; ++f) {
 	    const mesh::ind_pt_face_ct &fpts
@@ -676,7 +691,7 @@ struct Chrono {
 	    interpolate_face_chrono.tic();
 #endif
 	    
-	    interpolate_face(msh, ptdone, ipts,
+	    interpolate_face(pgt, msh, ptdone, ipts,
 			     msh.trans_of_convex(i)->structure()
 			     ->faces_structure()[f], fixed_points.size(),
 			     fixed_points_constraints, list_constraints);
@@ -704,7 +719,7 @@ struct Chrono {
       bgeot::pgeometric_trans pgt2 = bgeot::simplex_geotrans(n, K);
       papprox_integration
 	pai = classical_approx_im(pgt2,dim_type(2*K))->approx_method();
-      approx_integration new_approx(pgt->convex_ref());
+      auto new_approx = std::make_shared<approx_integration>(pgt->convex_ref());
       base_matrix KK(n,n), CS(n,n);
       base_matrix pc(pgt2->nb_points(), n); 
       for (dal::bv_visitor i(msh.convex_index()); !i.finished(); ++i) {
@@ -717,17 +732,16 @@ struct Chrono {
 	  pgt2->poly_vector_grad(pai->point(j), pc);
 	  gmm::mult(G,pc,KK);
 	  scalar_type J = gmm::lu_det(KK);
-	  if (noisy && J * sign < 0)
-	    cout << "ATTENTION retrournement de situation en convexe " << i
-		 << "sign = " << sign << " J = " << J << " p1 = "
-		 << msh.points_of_convex(i)[0] << " p2 = "
-		 << msh.points_of_convex(i)[1] << " p3 = "
-		 << msh.points_of_convex(i)[2] << " p4 = "
-		 << msh.points_of_convex(i)[3] << " p5 = "
-		 << msh.points_of_convex(i)[4] << " p6 = "
-		 << msh.points_of_convex(i)[5] << " K = " << int(K) << endl;
+	  if (noisy && J * sign < 0) {
+	    cout << "CAUTION reversal situation in convex " << i
+		 << "sign = " << sign << " J = " << J << endl;
+	    for (size_type nip = 0; nip < msh.nb_points_of_convex(i); ++nip)
+	      cout << "Point " << nip << "=" << msh.points_of_convex(i)[nip]
+		   << " ";
+	    cout << endl;
+	  }
 	  if (sign == 0 && gmm::abs(J) > 1E-14) sign = J;
-	  new_approx.add_point(c.xreal(), pai->coeff(j) * gmm::abs(J));
+	  new_approx->add_point(c.xreal(), pai->coeff(j) * gmm::abs(J));
 	}
       }
 
@@ -739,20 +753,20 @@ struct Chrono {
 						pai->point(0), G);
 	for (size_type j = 0; j < pai->nb_points_on_face(it.f()); ++j) {
 	  c.set_xref(pai->point_on_face(it.f(), j));
-	  new_approx.add_point(c.xreal(), pai->coeff_on_face(it.f(), j)
+	  new_approx->add_point(c.xreal(), pai->coeff_on_face(it.f(), j)
 			       * gmm::abs(c.J()), it.f() /* faux */);
 	}
       }
-      new_approx.valid_method();
+      new_approx->valid_method();
 
       /* 
        * Step 5 : Test the validity of produced integration method.
        */
       
-      scalar_type error = test_integration_error(&new_approx, 1);
+      scalar_type error = test_integration_error(new_approx, 1);
       if (noisy) cout << " max monomial integration err: " << error << "\n";
       if (error > 1e-5) {
-	if (noisy) cout << "PAS BON NON PLUS\n"; if (noisy) getchar();
+	if (noisy) cout << "Not Good ! Let us make a finer cut.\n"; if (noisy) getchar();
 	if (dmin > 3*h0) { dmin /= 2.; }
 	else { h0 /= 2.0; dmin = 2.*h0; }
 	h0_is_ok = false;
@@ -824,12 +838,14 @@ struct Chrono {
       mesh &msh = *(it->second.pmsh);      
       for (unsigned ils = 0; ils < nb_level_sets(); ++ils) {
 	if (get_level_set(ils)->has_secondary()) {
-	  mesher_level_set mesherls0 =  get_level_set(ils)->mls_of_convex(cv, 0, false);
-	  mesher_level_set mesherls1 =  get_level_set(ils)->mls_of_convex(cv, 1, false);
+	  pmesher_signed_distance
+	    mesherls0 = get_level_set(ils)->mls_of_convex(cv, 0, false);
+	  pmesher_signed_distance
+	    mesherls1 = get_level_set(ils)->mls_of_convex(cv, 1, false);
 	  for (dal::bv_visitor ii(msh.convex_index()); !ii.finished(); ++ii) {
 	    for (unsigned ipt = 0; ipt < msh.nb_points_of_convex(ii); ++ipt) {
-	      if (gmm::abs(mesherls0(msh.points_of_convex(ii)[ipt])) < 1E-10
-		  && gmm::abs(mesherls1(msh.points_of_convex(ii)[ipt])) < 1E-10) {
+	      if (gmm::abs((*mesherls0)(msh.points_of_convex(ii)[ipt])) < 1E-10
+		  && gmm::abs((*mesherls1)(msh.points_of_convex(ii)[ipt])) < 1E-10) {
 		crack_tip_convexes_.add(cv);
 		goto next_convex;
 	      }
@@ -917,17 +933,17 @@ struct Chrono {
 
     // cout << "cv " << cv << " radius = " << radius << endl;
 
-    mesher_level_set mls0 = ls->mls_of_convex(cv, 0), mls1(mls0);
+    pmesher_signed_distance mls0 = ls->mls_of_convex(cv, 0), mls1(mls0);
     if (ls->has_secondary()) mls1 = ls->mls_of_convex(cv, 1);
     int p = 0;
     bool is_cut = false;
     scalar_type d2 = 0, d1 = 1, d0 = 0, d0min = 0;
     for (size_type i = 0; i < pgt2->nb_points(); ++i) {
-      d0 = mls0(cvi.pmsh->points_of_convex(sub_cv)[i]);
+      d0 = (*mls0)(cvi.pmsh->points_of_convex(sub_cv)[i]);
       if (i == 0) d0min = gmm::abs(d0);
       else d0min = std::min(d0min, gmm::abs(d0));
       if (ls->has_secondary())
-	d1 = std::min(d1, mls1(cvi.pmsh->points_of_convex(sub_cv)[i]));
+	d1 = std::min(d1, (*mls1)(cvi.pmsh->points_of_convex(sub_cv)[i]));
      
       int p2 = ( (d0 < -EPS) ? -1 : ((d0 > EPS) ? +1 : 0));
       if (p == 0) p = p2;
@@ -963,23 +979,23 @@ struct Chrono {
       if (!p2 || p*p2 < 0) return 0;
     }
 
-    mesher_level_set mls1 = ls->mls_of_convex(cv, lsnum, false);
+    pmesher_signed_distance mls1 = ls->mls_of_convex(cv, lsnum, false);
     base_node X(pf->dim()), G(pf->dim());
     gmm::fill_random(X); X *= 1E-2;
-    scalar_type d = mls1.grad(X, G);
+    scalar_type d = mls1->grad(X, G);
     if (gmm::vect_norm2(G)*2.5 < gmm::abs(d)) return p;
 
     bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
-    std::unique_ptr<mesher_signed_distance> ref_element(new_ref_element(pgt));
+    pmesher_signed_distance ref_element = new_ref_element(pgt);
     
     gmm::fill_random(X); X *= 1E-2;
-    mesher_intersection mi1(*ref_element, mls1);
+    mesher_intersection mi1(ref_element, mls1);
     if (!try_projection(mi1, X)) return p;
     if ((*ref_element)(X) > 1E-8) return p;
     
     gmm::fill_random(X); X *= 1E-2;
-    mesher_level_set mls2 = ls->mls_of_convex(cv, lsnum, true);
-    mesher_intersection mi2(*ref_element, mls2);
+    pmesher_signed_distance mls2 = ls->mls_of_convex(cv, lsnum, true);
+    mesher_intersection mi2(ref_element, mls2);
     if (!try_projection(mi2, X)) return p;
     if ((*ref_element)(X) > 1E-8) return p;
    
